package com.maxiq.j2ee.jboss.deployers;

import java.util.Iterator;
import java.util.Set;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.jboss.beans.metadata.spi.BeanMetaData;
import org.jboss.deployers.spi.DeploymentException;
import org.jboss.deployers.spi.deployer.helpers.AbstractSimpleRealDeployer;
import org.jboss.deployers.structure.spi.DeploymentUnit;
import org.jboss.deployers.vfs.spi.structure.VFSDeploymentUnit;


/**
 * AxisRealDeployer
 * 
 * Deploy's the AxisMetaData to the AxisServer
 */
public class AxisRealDeployer extends AbstractSimpleRealDeployer<AxisMetaData> {

   public AxisRealDeployer() {
      super(AxisMetaData.class);
      setOutput(BeanMetaData.class);
   }

   public static MBeanServer getMBeanServer() {
      Iterator it = MBeanServerFactory.findMBeanServer(null).iterator();
      while (it.hasNext()) {
         MBeanServer aServer = (MBeanServer) it.next();
         if (isJBossServer(aServer))
            return aServer;
      }
      return null;
   }
   private static boolean isJBossServer(MBeanServer aServer) {
      if (aServer == null) return false;
      try {
         Set jmbSet = aServer.queryNames(new ObjectName("jboss.system:*"), null);
         return jmbSet != null && jmbSet.size() > 0;
      } catch (MalformedObjectNameException monex) {
         throw new RuntimeException(monex.getMessage());
      }
   }
   
   
   public void deploy(DeploymentUnit unit, AxisMetaData metaData) throws DeploymentException {
      if (unit instanceof VFSDeploymentUnit == false)
         return;
      
      MBeanServer server = getMBeanServer();
      if (server == null)
         throw new RuntimeException("No MBeanServer");
      try {
         ObjectName asixServiceObj = new ObjectName("jboss.net:service=Axis");
         Object[] params = {metaData.getWebServiceURL(), unit.getClassLoader()};
         String[] signature = {"java.net.URL", "java.lang.ClassLoader"};
         server.invoke(asixServiceObj, "deploy", params, signature);
      } catch (Exception ex) {
         throw new RuntimeException("Cannot deploy axis service. Cause: " + ex.getMessage());
      }
   }
}
